Type: Package
Package: lgr
Title: A Fully Featured Logging Framework
Version: 0.5.2
Authors@R: 
    person(given = "Stefan",
           family = "Fleck",
           role = c("aut", "cre"),
           email = "stefan.b.fleck@gmail.com",
           comment = c(ORCID = "0000-0003-3344-9851"))
Maintainer: Stefan Fleck <stefan.b.fleck@gmail.com>
Description: A flexible, light-weight logging
    framework based on 'R6' classes. It supports hierarchical loggers,
    structured logging, output to  plaintext, 'JSON', (rotating) files,
    and memory buffers. Additional appenders for 'Elasticsearch', 'Dynatrace', 
    databases, email and push notifications, and more are available
    in the 'lgrExtra' package.
License: MIT + file LICENSE
URL: https://s-fleck.github.io/lgr/
BugReports: https://github.com/s-fleck/lgr/issues/
Depends: R (>= 3.2.0)
Imports: R6 (>= 2.4.0)
Suggests: cli, covr, crayon, data.table, desc, future, future.apply,
        glue, jsonlite, knitr, rmarkdown, rotor (>= 0.3.5), rprojroot,
        testthat, tibble, tools, utils, whoami, yaml
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
Collate: 'Filterable.R' 'utils-sfmisc.R' 'utils.R' 'Appender.R'
        'Filter.R' 'log_levels.R' 'LogEvent.R' 'Layout.R'
        'LayoutJson.R' 'Logger.R' 'basic_config.R'
        'default_functions.R' 'event_list.R' 'get_logger.R'
        'lgr-package.R' 'logger_config.R' 'logger_index.R'
        'logger_tree.R' 'read_json_lines.R' 'simple_logging.R'
        'string_repr.R' 'use_logger.R' 'utils-formatting.R'
        'utils-logging.R' 'utils-rd.R' 'utils-rotor.R'
NeedsCompilation: no
Packaged: 2026-01-30 11:46:52 UTC; stefan.fleck
Author: Stefan Fleck [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3344-9851>)
Repository: CRAN
Date/Publication: 2026-01-30 12:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 12:08:19 UTC; unix
