# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,LogEvent)
S3method(as.data.frame,event_list)
S3method(as_LogEvent,data.frame)
S3method(as_LogEvent,list)
S3method(as_event_list,LogEvent)
S3method(as_event_list,data.frame)
S3method(as_event_list,list)
S3method(as_logger_config,"NULL")
S3method(as_logger_config,character)
S3method(as_logger_config,list)
S3method(format,LogEvent)
S3method(format,Logger)
S3method(format,ancestry)
S3method(format,logger_tree)
S3method(print,Appender)
S3method(print,LogEvent)
S3method(print,Logger)
S3method(print,ancestry)
S3method(print,logger_tree)
S3method(string_repr,"function")
S3method(string_repr,data.frame)
S3method(string_repr,default)
S3method(string_repr,matrix)
S3method(string_repr,numeric)
S3method(toString,LogEvent)
export(.obj)
export(Appender)
export(AppenderBuffer)
export(AppenderConsole)
export(AppenderFile)
export(AppenderFileRotating)
export(AppenderFileRotatingDate)
export(AppenderFileRotatingTime)
export(AppenderJson)
export(AppenderMemory)
export(AppenderTable)
export(CannotInitializeAbstractClassError)
export(EventFilter)
export(FilterForceLevel)
export(FilterInject)
export(Filterable)
export(Layout)
export(LayoutFormat)
export(LayoutGlue)
export(LayoutJson)
export(LogEvent)
export(Logger)
export(LoggerGlue)
export(add_appender)
export(add_log_levels)
export(as.data.table.event_list)
export(as_LogEvent)
export(as_event_list)
export(as_logger_config)
export(basic_config)
export(colorize_levels)
export(console_threshold)
export(default_exception_handler)
export(event_list)
export(get_caller)
export(get_log_levels)
export(get_logger)
export(get_logger_glue)
export(get_user)
export(is_filter)
export(is_log_level)
export(is_log_levels)
export(is_threshold)
export(label_levels)
export(lgr)
export(log_exception)
export(logger_index)
export(logger_tree)
export(pad_left)
export(pad_right)
export(read_json_lines)
export(remove_appender)
export(remove_log_levels)
export(show_data)
export(show_dt)
export(show_log)
export(standardize_log_level)
export(standardize_log_levels)
export(standardize_threshold)
export(string_repr)
export(suspend_logging)
export(threshold)
export(unlabel_levels)
export(unsuspend_logging)
export(use_logger)
export(with_log_level)
export(with_log_value)
export(with_logging)
export(without_logging)
importFrom(stats,setNames)
