---
title: "Example Configurations"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{lgr-simple}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "",
  echo = FALSE
)
```

```{r setup}
library(lgr)

```

# minimal

```{r}
cfg <- system.file("configs/minimal.yaml", package = "lgr")
cat(readLines(cfg), sep = "\n")

```
```{r}
print(lgr::lgr$config(cfg))
```
```{r}
lgr$info("test")
```



# recommended

```{r}
cfg <- system.file("configs/recommended.yaml", package = "lgr")
cat(readLines(cfg), sep = "\n")

```

```{r}
print(lgr::lgr$config(cfg))

```
```{r echo = TRUE, comment="#>"}
lgr$trace("trace messages get ignored by console appender")
lgr$info("test")

show_log()
```

