% Generated by roxygen2: do not edit by hand % Please edit documentation in R/Appender.R \name{AppenderFileRotatingDate} \alias{AppenderFileRotatingDate} \title{Log to a date-stamped rotating file} \description{ Log to a date-stamped rotating file Log to a date-stamped rotating file } \details{ This is a simpler version of AppenderFileRotatingTime when the timestamps do not need to include sub-day accuracy. } \seealso{ \link{AppenderFileRotatingTime}, \link{AppenderFileRotating}, \code{\link[rotor:rotate]{rotor::rotate()}} Other Appenders: \code{\link{Appender}}, \code{\link{AppenderBuffer}}, \code{\link{AppenderConsole}}, \code{\link{AppenderFile}}, \code{\link{AppenderFileRotating}}, \code{\link{AppenderFileRotatingTime}}, \code{\link{AppenderTable}} } \concept{Appenders} \section{Super classes}{ \code{\link[lgr:Filterable]{lgr::Filterable}} -> \code{\link[lgr:Appender]{lgr::Appender}} -> \code{\link[lgr:AppenderFile]{lgr::AppenderFile}} -> \code{\link[lgr:AppenderFileRotating]{lgr::AppenderFileRotating}} -> \code{\link[lgr:AppenderFileRotatingTime]{lgr::AppenderFileRotatingTime}} -> \code{AppenderFileRotatingDate} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-AppenderFileRotatingDate-new}{\code{AppenderFileRotatingDate$new()}} \item \href{#method-AppenderFileRotatingDate-clone}{\code{AppenderFileRotatingDate$clone()}} } } \if{html}{\out{
Inherited methods

- `lgr::Filterable$add_filter()`
- `lgr::Filterable$filter()`
- `lgr::Filterable$remove_filter()`
- `lgr::Filterable$set_filters()`
- `lgr::Appender$set_layout()`
- `lgr::Appender$set_threshold()`
- `lgr::AppenderFile$show()`
- `lgr::AppenderFileRotating$append()`
- `lgr::AppenderFileRotating$prune()`
- `lgr::AppenderFileRotating$set_backup_dir()`
- `lgr::AppenderFileRotating$set_compression()`
- `lgr::AppenderFileRotating$set_create_file()`
- `lgr::AppenderFileRotating$set_file()`
- `lgr::AppenderFileRotating$set_max_backups()`
- `lgr::AppenderFileRotating$set_size()`
- `lgr::AppenderFileRotatingTime$format()`
- `lgr::AppenderFileRotatingTime$rotate()`
- `lgr::AppenderFileRotatingTime$set_age()`
- `lgr::AppenderFileRotatingTime$set_cache_backups()`
- `lgr::AppenderFileRotatingTime$set_fmt()`
- `lgr::AppenderFileRotatingTime$set_overwrite()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-AppenderFileRotatingDate-new}{}}} \subsection{Method \code{new()}}{ \subsection{Usage}{ \if{html}{\out{
}}\preformatted{AppenderFileRotatingDate$new( file, threshold = NA_integer_, layout = LayoutFormat$new(), filters = NULL, age = Inf, size = -1, max_backups = Inf, compression = FALSE, backup_dir = dirname(file), fmt = "\%Y-\%m-\%d", overwrite = FALSE, cache_backups = TRUE, create_file = NULL )}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{size, age, max_backups, compression, backup_dir, fmt, overwrite, cache_backups}}{see \code{\link[rotor:rotate]{rotor::rotate_date()}} for the meaning of these arguments. Note that \code{fmt} corresponds to \code{format} (because \verb{$format} has a special meaning for R6 classes).} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-AppenderFileRotatingDate-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{AppenderFileRotatingDate$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }