% Generated by roxygen2: do not edit by hand % Please edit documentation in R/Appender.R \name{AppenderFileRotatingTime} \alias{AppenderFileRotatingTime} \title{Log to a time-stamped rotating file} \description{ Log to a time-stamped rotating file Log to a time-stamped rotating file } \details{ An extension of \link{AppenderFile} that rotates logfiles based on certain conditions. Please refer to the documentation of \code{\link[rotor:rotate]{rotor::rotate_time()}} for the meanings of the extra arguments } \seealso{ \link{AppenderFileRotatingDate}, \link{AppenderFileRotating}, \code{\link[rotor:rotate]{rotor::rotate()}} Other Appenders: \code{\link{Appender}}, \code{\link{AppenderBuffer}}, \code{\link{AppenderConsole}}, \code{\link{AppenderFile}}, \code{\link{AppenderFileRotating}}, \code{\link{AppenderFileRotatingDate}}, \code{\link{AppenderTable}} } \concept{Appenders} \section{Super classes}{ \code{\link[lgr:Filterable]{lgr::Filterable}} -> \code{\link[lgr:Appender]{lgr::Appender}} -> \code{\link[lgr:AppenderFile]{lgr::AppenderFile}} -> \code{\link[lgr:AppenderFileRotating]{lgr::AppenderFileRotating}} -> \code{AppenderFileRotatingTime} } \section{Active bindings}{ \if{html}{\out{
}} \describe{ \item{\code{cache_backups}}{\code{TRUE} or \code{FALSE}. If \code{TRUE} (the default) the list of backups is cached, if \code{FALSE} it is read from disk every time this appender triggers. Caching brings a significant speedup for checking whether to rotate or not based on the \code{age} of the last backup, but is only safe if there are no other programs/functions (except this appender) interacting with the backups.} } \if{html}{\out{
}} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-AppenderFileRotatingTime-new}{\code{AppenderFileRotatingTime$new()}} \item \href{#method-AppenderFileRotatingTime-rotate}{\code{AppenderFileRotatingTime$rotate()}} \item \href{#method-AppenderFileRotatingTime-set_age}{\code{AppenderFileRotatingTime$set_age()}} \item \href{#method-AppenderFileRotatingTime-set_fmt}{\code{AppenderFileRotatingTime$set_fmt()}} \item \href{#method-AppenderFileRotatingTime-set_overwrite}{\code{AppenderFileRotatingTime$set_overwrite()}} \item \href{#method-AppenderFileRotatingTime-set_cache_backups}{\code{AppenderFileRotatingTime$set_cache_backups()}} \item \href{#method-AppenderFileRotatingTime-format}{\code{AppenderFileRotatingTime$format()}} \item \href{#method-AppenderFileRotatingTime-clone}{\code{AppenderFileRotatingTime$clone()}} } } \if{html}{\out{
Inherited methods

- `lgr::Filterable$add_filter()`
- `lgr::Filterable$filter()`
- `lgr::Filterable$remove_filter()`
- `lgr::Filterable$set_filters()`
- `lgr::Appender$set_layout()`
- `lgr::Appender$set_threshold()`
- `lgr::AppenderFile$show()`
- `lgr::AppenderFileRotating$append()`
- `lgr::AppenderFileRotating$prune()`
- `lgr::AppenderFileRotating$set_backup_dir()`
- `lgr::AppenderFileRotating$set_compression()`
- `lgr::AppenderFileRotating$set_create_file()`
- `lgr::AppenderFileRotating$set_file()`
- `lgr::AppenderFileRotating$set_max_backups()`
- `lgr::AppenderFileRotating$set_size()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-AppenderFileRotatingTime-new}{}}} \subsection{Method \code{new()}}{ \subsection{Usage}{ \if{html}{\out{
}}\preformatted{AppenderFileRotatingTime$new( file, threshold = NA_integer_, layout = LayoutFormat$new(), filters = NULL, age = Inf, size = -1, max_backups = Inf, compression = FALSE, backup_dir = dirname(file), fmt = "\%Y-\%m-\%d--\%H-\%M-\%S", overwrite = FALSE, cache_backups = TRUE, create_file = NULL )}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{size, age, max_backups, compression, backup_dir, fmt, overwrite, cache_backups}}{see \code{\link[rotor:rotate]{rotor::rotate_time()}} for the meaning of these arguments. Note that \code{fmt} corresponds to \code{format} and \code{backup_dir} to \code{dir}.} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-AppenderFileRotatingTime-rotate}{}}} \subsection{Method \code{rotate()}}{ \subsection{Usage}{ \if{html}{\out{
}}\preformatted{AppenderFileRotatingTime$rotate(force = FALSE, now = Sys.time())}\if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-AppenderFileRotatingTime-set_age}{}}} \subsection{Method \code{set_age()}}{ \subsection{Usage}{ \if{html}{\out{
}}\preformatted{AppenderFileRotatingTime$set_age(x)}\if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-AppenderFileRotatingTime-set_fmt}{}}} \subsection{Method \code{set_fmt()}}{ \subsection{Usage}{ \if{html}{\out{
}}\preformatted{AppenderFileRotatingTime$set_fmt(x)}\if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-AppenderFileRotatingTime-set_overwrite}{}}} \subsection{Method \code{set_overwrite()}}{ \subsection{Usage}{ \if{html}{\out{
}}\preformatted{AppenderFileRotatingTime$set_overwrite(x)}\if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-AppenderFileRotatingTime-set_cache_backups}{}}} \subsection{Method \code{set_cache_backups()}}{ set the \code{cache_backups} flag. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{AppenderFileRotatingTime$set_cache_backups(x)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{x}}{a \code{logical} scalar} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-AppenderFileRotatingTime-format}{}}} \subsection{Method \code{format()}}{ \subsection{Usage}{ \if{html}{\out{
}}\preformatted{AppenderFileRotatingTime$format(color = FALSE, ...)}\if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-AppenderFileRotatingTime-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{AppenderFileRotatingTime$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }